@@ -46,9 +46,12 @@ In-depth explanations of the layer components can be found in the module
 documentation for TileStache.Providers, TileStache.Core, and TileStache.Geography.
 """
 
+import sys
 from sys import stderr, modules
 from os.path import realpath, join as pathjoin
 from urlparse import urljoin, urlparse
+from mimetypes import guess_type
+from urllib import urlopen
 from json import dumps
 
 try:
@@ -97,11 +100,18 @@ class Configuration:
           dirpath:
             Local filesystem path for this configuration,
             useful for expanding relative paths.
+          
+        Optional attribute:
+        
+          index:
+            Mimetype, content tuple for default index response.
     """
     def __init__(self, cache, dirpath):
         self.cache = cache
         self.dirpath = dirpath
         self.layers = {}
+        
+        self.index = 'text/plain', 'TileStache bellows hello.'
 
 class Bounds:
     """ Coordinate bounding box for tiles.
@@ -159,8 +169,14 @@ def buildConfiguration(config_dict, dirpath='.'):
     
         The second argument is an optional dirpath that specifies where in the
         local filesystem the parsed dictionary originated, to make it possible
-        to resolve relative paths.
+        to resolve relative paths. It might be a path or more likely a full
+        URL including the "file://" prefix.
     """
+    scheme, h, path, p, q, f = urlparse(dirpath)
+    
+    if scheme in ('', 'file'):
+        sys.path.insert(0, path)
+    
     cache_dict = config_dict.get('cache', {})
     cache = _parseConfigfileCache(cache_dict, dirpath)
     
@@ -169,6 +185,13 @@ def buildConfiguration(config_dict, dirpath='.'):
     for (name, layer_dict) in config_dict.get('layers', {}).items():
         config.layers[name] = _parseConfigfileLayer(layer_dict, config, dirpath)
 
+    if 'index' in config_dict:
+        index_href = urljoin(dirpath, config_dict['index'])
+        index_body = urlopen(index_href).read()
+        index_type = guess_type(index_href)
+        
+        config.index = index_type[0], index_body
+    
     return config
 
 def enforcedLocalPath(relpath, dirpath, context='Path'):
@@ -295,6 +318,9 @@ def _parseConfigfileLayer(layer_dict, config, dirpath):
     if 'allowed origin' in layer_dict:
         layer_kwargs['allowed_origin'] = str(layer_dict['allowed origin'])
     
+    if 'maximum cache age' in layer_dict:
+        layer_kwargs['max_cache_age'] = int(layer_dict['maximum cache age'])
+    
     if 'preview' in layer_dict:
         preview_dict = layer_dict['preview']
         
