@@ -73,8 +73,9 @@ import os
 
 from StringIO import StringIO
 from posixpath import exists
+from thread import allocate_lock
 from urlparse import urlparse, urljoin
-from httplib import HTTPConnection
+import urllib
 from tempfile import mkstemp
 from string import Template
 from urllib import urlopen
@@ -99,9 +100,9 @@ except ImportError:
 import ModestMaps
 from ModestMaps.Core import Point, Coordinate
 
+import Vector
 import MBTiles
 import Geography
-import Vector
 
 def getProviderByName(name):
     """ Retrieve a provider object by name.
@@ -177,11 +178,7 @@ class Proxy:
         urls = self.provider.getTileUrls(coord)
         
         for url in urls:
-            s, host, path, p, query, f = urlparse(url)
-            conn = HTTPConnection(host, 80)
-            conn.request('GET', path+'?'+query)
-
-            body = conn.getresponse().read()
+            body = urllib.urlopen(url).read()
             tile = Image.open(StringIO(body)).convert('RGBA')
 
             if len(urls) == 1:
@@ -199,7 +196,9 @@ class Proxy:
             img.paste(tile, (0, 0), tile)
         
         return img
-            
+
+global_mapnik_lock = allocate_lock()
+
 class Mapnik:
     """ Built-in Mapnik provider. Renders map images from Mapnik XML files.
     
@@ -256,12 +255,17 @@ class Mapnik:
                 mapnik.load_map(self.mapnik, filename)
                 os.unlink(filename)
         
-        self.mapnik.width = width
-        self.mapnik.height = height
-        self.mapnik.zoom_to_box(mapnik.Envelope(xmin, ymin, xmax, ymax))
-        
-        img = mapnik.Image(width, height)
-        mapnik.render(self.mapnik, img)
+        #
+        # Mapnik can behave strangely when run in threads, so place a lock on the instance.
+        #
+        if global_mapnik_lock.acquire():
+            self.mapnik.width = width
+            self.mapnik.height = height
+            self.mapnik.zoom_to_box(mapnik.Envelope(xmin, ymin, xmax, ymax))
+            
+            img = mapnik.Image(width, height)
+            mapnik.render(self.mapnik, img)
+            global_mapnik_lock.release()
         
         img = Image.fromstring('RGBA', (width, height), img.tostring())
         
