@@ -8,17 +8,22 @@ designers and cartographers.
 
 Documentation available at http://tilestache.org/doc/
 """
-__version__ = '1.21.3'
+__version__ = '1.27.1'
 
 import re
 
 from sys import stdout
-from cgi import parse_qs
+try:
+    from urlparse import parse_qs
+except ImportError:
+    from cgi import parse_qs
 from StringIO import StringIO
 from os.path import dirname, join as pathjoin, realpath
+from datetime import datetime, timedelta
 from urlparse import urljoin, urlparse
 from urllib import urlopen
 from os import getcwd
+from time import time
 
 try:
     from json import load as json_load
@@ -33,9 +38,12 @@ import Config
 # dictionary of configuration objects for requestLayer().
 _previous_configs = {}
 
+# dictionary of tiles seen recently in this process, when ignore_cached is on.
+_recent_tiles = {}
+
 # regular expression for PATH_INFO
 _pathinfo_pat = re.compile(r'^/?(?P<l>\w.+)/(?P<z>\d+)/(?P<x>-?\d+)/(?P<y>-?\d+)\.(?P<e>\w+)$')
-_preview_pat = re.compile(r'^/?(?P<l>\w.+)/preview\.html$')
+_preview_pat = re.compile(r'^/?(?P<l>\w.+)/(preview\.html)?$')
 
 def getTile(layer, coord, extension, ignore_cached=False):
     """ Get a type string and tile binary for a given request layer tile.
@@ -51,10 +59,21 @@ def getTile(layer, coord, extension, ignore_cached=False):
     """
     mimetype, format = layer.getTypeByExtension(extension)
     cache = layer.config.cache
+
+    # key in _recent_tiles
+    _tile = (layer, coord, extension)
     
     if not ignore_cached:
         # Start by checking for a tile in the cache.
         body = cache.read(layer, coord, format)
+
+    elif _tile in _recent_tiles:
+        # Then look in the bag of recent tiles.
+        body, use_by = _recent_tiles[_tile]
+        if time() > use_by:
+            del _recent_tiles[_tile]
+            body = None
+
     else:
         # Bypass the cache
         body = None
@@ -108,6 +127,9 @@ def getTile(layer, coord, extension, ignore_cached=False):
                 # Always clean up a lock when it's no longer being used.
                 cache.unlock(layer, lockCoord, format)
     
+    if ignore_cached:
+        _recent_tiles[_tile] = body, time() + 300
+    
     return mimetype, body
 
 def getPreview(layer):
@@ -153,12 +175,15 @@ def splitPathInfo(pathinfo):
         
         Example: "/layer/0/0/0.png", leading "/" optional.
     """
-    if _pathinfo_pat.match(pathinfo):
+    if pathinfo == '/':
+        return None, None, None
+    
+    if _pathinfo_pat.match(pathinfo or ''):
         path = _pathinfo_pat.match(pathinfo)
         layer, row, column, zoom, extension = [path.group(p) for p in 'lyxze']
         coord = Coordinate(int(row), int(column), int(zoom))
 
-    elif _preview_pat.match(pathinfo):
+    elif _preview_pat.match(pathinfo or ''):
         path = _preview_pat.match(pathinfo)
         layer, extension = path.group('l'), 'html'
         coord = None
@@ -197,6 +222,12 @@ def requestLayer(config, path_info):
         assert hasattr(config, 'layers'), 'Configuration object must have layers.'
         assert hasattr(config, 'dirpath'), 'Configuration object must have a dirpath.'
     
+    # ensure that path_info is at least a single "/"
+    path_info = '/' + (path_info or '').lstrip('/')
+    
+    if path_info == '/':
+        return Core.Layer(config, None, None)
+
     layername = splitPathInfo(path_info)[0]
     
     if layername not in config.layers:
@@ -204,31 +235,48 @@ def requestLayer(config, path_info):
     
     return config.layers[layername]
 
-def requestHandler(config, path_info, query_string):
+def requestHandler(config_hint, path_info, query_string):
     """ Generate a mime-type and response body for a given request.
     
         Requires a configuration and PATH_INFO (e.g. "/example/0/0/0.png").
         
-        Config parameter can be a file path string for a JSON configuration file
+        Config_hint parameter can be a path string for a JSON configuration file
         or a configuration object with 'cache', 'layers', and 'dirpath' properties.
         
-        Query string is optional and not currently used. Calls getTile()
-        to render actual tiles, and getPreview() to render preview.html.
+        Query string is optional, currently used for JSON callbacks.
+        
+        Calls getTile() to render actual tiles, and getPreview() to render preview.html.
     """
     try:
-        if path_info is None:
-            raise Core.KnownUnknown('Missing path_info in requestHandler().')
-    
-        layer = requestLayer(config, path_info)
-        query = parse_qs(query_string or '')
+        # ensure that path_info is at least a single "/"
+        path_info = '/' + (path_info or '').lstrip('/')
         
+        layer = requestLayer(config_hint, path_info)
+        query = parse_qs(query_string or '')
+        try:
+            callback = query['callback'][0]
+        except KeyError:
+            callback = None
+        
+        #
+        # Special case for index page.
+        #
+        if path_info == '/':
+            return getattr(layer.config, 'index', ('text/plain', 'TileStache says hello.'))
+
         coord, extension = splitPathInfo(path_info)[1:]
         
-        if extension == 'html' and coord is None:
+        if path_info == '/':
+            raise Exception(path_info)
+        
+        elif extension == 'html' and coord is None:
             mimetype, content = getPreview(layer)
 
         else:
             mimetype, content = getTile(layer, coord, extension)
+    
+        if callback and 'json' in mimetype:
+            mimetype, content = 'application/javascript', '%s(%s)' % (callback, content)
 
     except Core.KnownUnknown, e:
         out = StringIO()
@@ -263,6 +311,11 @@ def cgiHandler(environ, config='./tilestache.cfg', debug=False):
     if layer.allowed_origin:
         print >> stdout, 'Access-Control-Allow-Origin:', layer.allowed_origin
     
+    if layer.max_cache_age is not None:
+        expires = datetime.utcnow() + timedelta(seconds=layer.max_cache_age)
+        print >> stdout, 'Expires:', expires.strftime('%a %d %b %Y %H:%M:%S GMT')
+        print >> stdout, 'Cache-Control: public, max-age=%d' % layer.max_cache_age
+    
     print >> stdout, 'Content-Length: %d' % len(content)
     print >> stdout, 'Content-Type: %s\n' % mimetype
     print >> stdout, content
@@ -319,14 +372,16 @@ class WSGITileServer:
         except Core.KnownUnknown, e:
             return self._response(start_response, '400 Bad Request', str(e))
 
-        if layer not in self.config.layers:
+        if layer and layer not in self.config.layers:
             return self._response(start_response, '404 Not Found')
 
         mimetype, content = requestHandler(self.config, environ['PATH_INFO'], environ['QUERY_STRING'])
-        allowed_origin = requestLayer(self.config, environ['PATH_INFO']).allowed_origin
-        return self._response(start_response, '200 OK', str(content), mimetype, allowed_origin)
+        request_layer = requestLayer(self.config, environ['PATH_INFO'])
+        allowed_origin = request_layer.allowed_origin
+        max_cache_age = request_layer.max_cache_age
+        return self._response(start_response, '200 OK', str(content), mimetype, allowed_origin, max_cache_age)
 
-    def _response(self, start_response, code, content='', mimetype='text/plain', allowed_origin=''):
+    def _response(self, start_response, code, content='', mimetype='text/plain', allowed_origin='', max_cache_age=None):
         """
         """
         headers = [('Content-Type', mimetype), ('Content-Length', str(len(content)))]
@@ -334,6 +389,11 @@ class WSGITileServer:
         if allowed_origin:
             headers.append(('Access-Control-Allow-Origin', allowed_origin))
         
+        if max_cache_age is not None:
+            expires = datetime.utcnow() + timedelta(seconds=max_cache_age)
+            headers.append(('Expires', expires.strftime('%a %d %b %Y %H:%M:%S GMT')))
+            headers.append(('Cache-Control', 'public, max-age=%d' % max_cache_age))
+        
         start_response(code, headers)
         return [content]
 
